#!/bin/bash
#
# requires: ...topnm, pnmscale, pnmcomp, ppmntsc, ppmtoy4m, mpeg2enc
#

# video format. pal or ntsc
FORMAT=pal

# target image width/height (taking into account visible screen area)
##if [ "$FORMAT" = "ntsc" ]; then
##  TW=600
##  TH=420
##else
##  TW=632
##  TH=512
##fi

if [ "$FORMAT" = "ntsc" ]; then
  TW=600
  TH=420
else
  TW=704
  TH=576
fi




## ONLY CHANGE ANYTHING IF YOU KNOW WHAT YOU DO
TMP=/tmp/image_convert.$$.pnm

## DON'T CHANGE /FILE TO CONVERT
IMG=$1

## DON'T CHANGE
MPG=$2

## DON'T CHANGE
THEMECACHEDIR=$3

## DON'T CHANGE
COVERXY=$4

##DON'T CHANGE
COVERSIZE=$5

DIR=`dirname "$MPG"`
if [ ! -d "$DIR" ]; then
  mkdir -p "$DIR"
fi

#
# get the file type and set the according converter to PNM
#
FILE_TYPE=`file -i -L -b "$IMG" 2>/dev/null | cut -f2 -d/`
case "$FILE_TYPE" in
  jpg | jpeg)
  TO_PNM=jpegtopnm
  ;;
  tiff)
  TO_PNM=tifftopnm
  ;;
  bmp | x-bmp)
  TO_PNM=bmptoppm
  ;;
  png | x-png)
  TO_PNM=pngtopnm
  ;;
  Netpbm | pnm | x-portable-pixmap)
  TO_PNM=cat
  ;;
  gif)
  TO_PNM=giftopnm
  ;;
  *)
  echo "filetype '$FILE_TYPE' is not supported"
  exit 1
  ;;
esac
#
# extract the image size & compute scale value
#
LANG=C # get the decimal point right

#
# resize the image and composite the background
#
IMG2=/tmp/resize-background.$FILE_TYPE
COVIMG=$THEMECACHEDIR/background.$FILE_TYPE
COMPIMG=/tmp/composite.$FILE_TYPE


convert -resize $COVERSIZE! "$IMG" "$IMG2"

composite -geometry $COVERXY "$IMG2" "$COVIMG" "$COMPIMG" 


$TO_PNM "$COMPIMG" >$TMP 2>/dev/null

S=`pnmfile $TMP | awk '{ printf "%d %d ",$4,$6 }'`
S=`echo $S $TW $TH | awk '{ sw=$3/$1; sh=$4/$2; s=(sw<sh)?sw:sh; printf "%.4f\n",(s>1)?1.0:s; }'`
#
# now run the conversion
#
if [ "$FORMAT" = "ntsc" ]; then
  pnmscale -xysize 704 480 $TMP | \
#    pnmpad -black -width 704 -height 480 | \
    ppmntsc | \
    ppmtoy4m -v 0 -n 1 -r -F 30000:1001 | \
    mpeg2enc -f 7 -T 90 -F 4 -nn -a 2 -v 0 -o "$MPG"
else
  pnmscale -xysize 704 576  $TMP | \
#    pnmpad -black -width 704 -height 576 | \
    ppmntsc --pal | \
    ppmtoy4m -v 0 -n 1 -r -F 25:1 | \
    mpeg2enc -f 7 -T 90 -F 3 -np -a 2 -v 2 -o "$MPG"
fi
#
# cleanup
#
rm $TMP
rm $IMG2
rm $COMPIMG
