/*
 * MP3/MPlayer plugin to VDR (C++)
 *
 * (C) 2001-2005 Stefan Huelswitt <s.huelswitt@gmx.de>
 *
 * This code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * Or, point your browser to http://www.gnu.org/copyleft/gpl.html
 */

#include <string.h>

#include "common.h"
#include "setup-mp3.h"

cMP3Setup MP3Setup;

// --- cMP3Setup ---------------------------------------------------------------

cMP3Setup::cMP3Setup(void)
{
  InitLoopMode = 0;
  InitShuffleMode = 0;
  AudioMode = 1;
  BgrScan = 2;
  EditorMode = 1;
  DisplayMode = 3;
  BackgrMode = 1;
  MenuMode = 2;
  TargetLevel = DEFAULT_TARGET_LEVEL;
  LimiterLevel = DEFAULT_LIMITER_LEVEL;
  Only48kHz = 0;
  UseProxy = 0;
  strcpy(ProxyHost,"localhost");
  ProxyPort = 8080;
  UseCddb = 1;
  strcpy(CddbHost,"freedb.freedb.org");
  CddbPort = 888;
  AudioOutMode = 0;
  AbortAtEOL = 1;
  ReplayDisplay = 0;
  HideMainMenu = 0;
  KeepSelect = 0;
  strcpy(CopyDir,"/tmp");
  strcpy(ImageCacheDir,"/var/cache/images");
  osdtheme = 1;
  CanScroll = 0;
  CanSkip = 0;
  Skiptime = 30;
  Jumptime = 10;
  OSDoffsetx = 0;
  OSDoffsety = 0;
}

const cMP3Theme mp3Theme[eMP3ThemeMaxNumber] =
{
  {
    // Future1 OK
/*    0x0A000000, //clrBackground
    0xFF000000, //clrListText
    0xFF000000, //clrTrackTitel
    0xFF000000, //clrItemText
    0xBC000000, //clrItemActive
    0xFFEEECEC, //clrItemActiveText
    0xFFFDDA15, //clrProgress
    0xFF000000, //clrProgressBackground
    0xFFF4F2F0, //clrActive
    0xFFA05A0E, //clrInactive
    "Future1",  //ID
    "/var/cache/images/mp3-MKIII-Future1",  // CacheDir
     0,        //ListOffsetX
     0,        //ListOffsetY
     566,       //ListWidth
     0,        //SymbolOffsetX
     1,         //SymbolOffsetY ">=1"
     0,        //InfoOffsetX
     1,        //InfoOffsetY ">=1"
     430,       //InfoWidth
     0,	//ProgressOffsetX
     1,		//ProgressOffsetY ">=1"
     420,       //ProgressWidth
     10,        //ProgressHeight
     6,         //RowCount  */
    0x0A000000, //clrBackground
    0xFF000000, //clrListText
    0xFF000000, //clrTrackTitel
    0xFF000000, //clrItemText
    0xBC000000, //clrItemActive
    0xFFEEECEC, //clrItemActiveText
    0xFFFDDA15, //clrProgress
    0xFF000000, //clrProgressBackground
    0xFFF4F2F0, //clrActive
    0xFFA05A0E, //clrInactive
    "Future1",  //ID
     24,        //ListOffsetX
     80,        //ListOffsetY
     566,       //ListWidth
     22,        //SymbolOffsetX
     70,        //SymbolOffsetY ">=1"
     22,        //InfoOffsetX
     2,         //InfoOffsetY ">=1"
     430,       //InfoWidth
     22,	//ProgressOffsetX
     6,		//ProgressOffsetY ">=1"
     420,       //ProgressWidth
     10,        //ProgressHeight
     "+513+341",//CoverXY
     "128x128",	//CoverSize
     6,         //RowCount
  },
  { // Audioplayer  OK
    0x0A000000, //clrBackground
    0xFFDFF0FF, //clrListText
    0xFFE2DCCE, //clrTrackTitel
    0xFF000000, //clrItemText
    0xBC000000, //clrItemActive
    0xFFCE7B00, //clrItemActiveText
    0xFFFFFF00, //clrProgress
    0xFF000000, //clrProgressBackground
    0xFFFE9B3A, //clrActive
    0xFFACA2A2, //clrInactive
    "Audioplayer",  //ID
     24,        //ListOffsetX
     82,        //ListOffsetY
     566,       //ListWidth
     42,        //SymbolOffsetX
     28,        //SymbolOffsetY ">=1"
     42,        //InfoOffsetX
     1,         //InfoOffsetY ">=1"
     440,       //InfoWidth
     42,	//ProgressOffsetX
     1,		//ProgressOffsetY ">=1"
     430,       //ProgressWidth
     10,        //ProgressHeight
     "+548+449",//CoverXY
     "75x75",	//CoverSize
     10,        //RowCount
  },
  { // Ipod OK
    0x0A000000, //clrBackground
    0xFF7F909F, //clrListText
    0xFF506F86, //clrTrackTitel
    0xFF000000, //clrItemText
    0xBC8EA4B0, //clrItemActive
    0xFFDDE3E7, //clrItemActiveText
    0xFFF7A683, //clrProgress
    0xFF000000, //clrProgressBackground
    0xFFF4F2F0, //clrActive
    0xFF949785, //clrInactive
    "Ipod",  //ID
     52,        //ListOffsetX
     48,        //ListOffsetY
     516,       //ListWidth
     52,        //SymbolOffsetX
     4,         //SymbolOffsetY ">=1"
     52,        //InfoOffsetX
     1,         //InfoOffsetY ">=1"
     516,       //InfoWidth
     52,	//ProgressOffsetX
     1,		//ProgressOffsetY ">=1"
     506,       //ProgressWidth
     8,         //ProgressHeight
     "+86+399",	//CoverXY
     "128x128",	//CoverSize
     7,         //RowCount
  },
  { // Itunes OK
    0x0A000000, //clrBackground
    0xFF4F575F, //clrListText
    0xFF4D5230, //clrTrackTitel
    0xFF4D5230, //clrItemText
    0xFF808080, //clrItemActive
    0xFFFFFFFF, //clrItemActiveText
    0xFFF7A683, //clrProgress
    0xFF000000, //clrProgressBackground
    0xFFCE6100, //clrActive
    0xFF949785, //clrInactive
    "Itunes",  //ID
     24,        //ListOffsetX
     164,       //ListOffsetY
     566,       //ListWidth
     34,        //SymbolOffsetX
     12,        //SymbolOffsetY ">=1"
     34,        //InfoOffsetX
     16,        //InfoOffsetY ">=1"
     550,       //InfoWidth
     34,	//ProgressOffsetX
     2,		//ProgressOffsetY ">=1"
     540,	//ProgressWidth
     10,	//ProgressHeight
     "+285+38",	//CoverXY
     "128x128",	//CoverSize
     7,         //RowCount
  },
  {
    // Kleo
    0x0A000000, //clrBackground
    0xFFE5EBEB, //clrListText
    0xFFFFFFC0, //clrTrackTitel
    0xFFC9D5E7, //clrItemText
    0xFF000000, //clrItemActive
    0xFFD8EAEC, //clrItemActiveText
//    0xA06C9ABD, //clrProgress
    0x0A000000, //clrProgress
    0x7F000000, //clrProgressBackground
//    0xFFF58649, //clrActive
    0xFF000000, //clrActive
    0x2F000000, //clrInactive
    "Kleo",  //ID
     28,        //ListOffsetX
     60,        //ListOffsetY
     566,       //ListWidth
     347,       //SymbolOffsetX
     36,        //SymbolOffsetY ">=1"
     34,        //InfoOffsetX
     31,        //InfoOffsetY ">=1"
     390,       //InfoWidth
     15,	//ProgressOffsetX
     13,	//ProgressOffsetY ">=1"
     404,	//ProgressWidth
     16,	//ProgressHeight
     "+506+332",//CoverXY
     "136x136",	//CoverSize
     7,         //RowCount
  },
};

